/***************************************************************************
 *   Copyright (C) 2004 by Olivier Galizzi                                 *
 *   olivier.galizzi@imag.fr                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SDEC_MOVING_WALL_HPP
#define SDEC_MOVING_WALL_HPP

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

#include <yade/yade-core/FileGenerator.hpp>

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

class SDECMovingWall : public FileGenerator
{
	private : Vector3r nbSpheres;
	private : bool useSpheresAsGround, side1wire, side2wire, side3wire;
	private : Real minRadius,density;
	private : Vector3r groundSize,groundPosition,wallSize,wallPosition,wallTranslationAxis,gravity;
	private : Vector3r side1Size,side1Position,side2Size,side2Position,side3Size,side3Position;
	private : Real maxRadius,wallVelocity;
	private : Real dampingForce;
	private	: Real disorder;
	private : Real dampingMomentum;
	private : Real spheresHeight;

	private : int timeStepUpdateInterval;
	private : bool rotationBlocked;
	private : Real sphereYoungModulus,spherePoissonRatio,sphereFrictionDeg;
	// construction
	public : SDECMovingWall ();
	public : ~SDECMovingWall ();


	private : void createBox(shared_ptr<Body>& body, Vector3r position, Vector3r extents, bool wire);
	private : void createSphere(shared_ptr<Body>& body, int i, int j, int k);
	private : void createGroundSphere(shared_ptr<Body>& body,Real radius, Real i, Real j, Real k);
	private : void createActors(shared_ptr<MetaBody>& rootBody);
	private : void positionRootBody(shared_ptr<MetaBody>& rootBody);

	protected : virtual void postProcessAttributes(bool deserializing);
	public : void registerAttributes();

	public : string generate();

	REGISTER_CLASS_NAME(SDECMovingWall);
	REGISTER_BASE_CLASS_NAME(FileGenerator);

};

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

REGISTER_SERIALIZABLE(SDECMovingWall,false);

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

#endif // SDEC_MOVING_WALL_HPP 

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////

